import 'package:flutter/material.dart';
import 'package:youtube_explode_dart/youtube_explode_dart.dart';
import 'package:path_provider/path_provider.dart';
import 'dart:io';
import 'package:gallery_saver/gallery_saver.dart';
import 'package:permission_handler/permission_handler.dart';

class VideoDownloadPage extends StatefulWidget {
  @override
  _VideoDownloadPageState createState() => _VideoDownloadPageState();
}

class _VideoDownloadPageState extends State<VideoDownloadPage> {
  final TextEditingController _urlController = TextEditingController();
  String _message = "";
  final YoutubeExplode _yt = YoutubeExplode();

  Future<void> downloadFile(String url) async {
    try {
      setState(() {
        _message = "Please wait...";
      });
      var video = await _yt.videos.get(url);
      var manifest = await _yt.videos.streamsClient.getManifest(video.id);
      var streamInfo = manifest.muxed.withHighestBitrate();

      final Directory tempDir = await getTemporaryDirectory();
      final File file = await File('${tempDir.path}/temp.mp4').create();
      var fileSink = file.openWrite();

      var stream = _yt.videos.streamsClient.get(streamInfo);
      await stream.pipe(fileSink);

      await fileSink.flush();
      await fileSink.close();

      final status = await Permission.storage.request();
      if (status.isGranted) {
        GallerySaver.saveVideo(file.path).then((bool? success) {
          setState(() {
            if (success == true) {
              _message = "Video saved successfully in gallery.";
            } else {
              _message = "Error occurred while saving video in gallery.";
            }
          });
        });
      } else {
        setState(() {
          _message = "Permission not granted. Can't save video.";
        });
      }
    } catch (e) {
      setState(() {
        _message = "An error occurred: $e";
      });
    } finally {
      _yt.close();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('Download Video'),
      ),
      body: Padding(
        padding: const EdgeInsets.all(8.0),
        child: Column(
          children: [
            TextField(
              controller: _urlController,
              decoration: InputDecoration(
                labelText: "Enter URL",
              ),
            ),
            Padding(
              padding: const EdgeInsets.all(100.0),
              child: ElevatedButton(
                onPressed: () async {
                  await downloadFile(_urlController.text);
                  // final snackBar = SnackBar(content: Text(_message));
                  // ScaffoldMessenger.of(context).showSnackBar(snackBar);
                },
                child: Text('Download'),
              ),
            ),
            if (_message.isNotEmpty)
              Padding(
                padding: const EdgeInsets.only(top: 30.0),
                child: Text(_message),
              ),
          ],
        ),
      ),
    );
  }
}
